<?php

/**
 * @file
 * Defines Drupal\lingotek\LingotekApi
 */

class LingotekApi {
  /**
   * The status string representing a successful API call.
   */
  const RESPONSE_STATUS_SUCCESS = 'success';

  /**
   * The server name of the Lingotek production instance.
   */
  const LINGOTEK_SERVER_PRODUCTION = 'http://myaccount.lingotek.com';
  //const LINGOTEK_SERVER_PRODUCTION = 'http://cms.lingotek.com';
  //const LINGOTEK_SERVER_PRODUCTION = 'http://10.0.11.220:8080';

  /**
   * The faux Lingotek user ID to use for anonymous user operations.
   */
  const ANONYMOUS_LINGOTEK_ID = 'anonymous';

  /**
   * The endpoint for API version 4
   */
  const API_ENDPOINT_V4 = '/lingopoint/api/4';

  /**
   * Holds the static instance of the singleton object.
   *
   * @var LingotekApi
   */
  private static $instance;

  /**
   * Debug status for extra logging of API calls.
   *
   * @var bool
   */
  private $debug;

  /**
   * The endpoint for API calls.
   *
   * @var string
   */
  private $api_url;

  /**
   * Gets the singleton instance of the API class.
   *
   * @return LingotekApi
   *   An instantiated LingotekApi object.
   */
  public static function instance() {
    if (!isset(self::$instance)) {
      $class_name = __CLASS__;
      self::$instance = new $class_name();
    }

    return self::$instance;
  }

  /**
   * Add a document to the Lingotek platform.
   *
   * Uploads the node's field content in the node's selected language.
   *
   * @param object $node
   *   A Drupal node object.
   */
  public function addContentDocument($node) {
    global $_lingotek_locale;
    $success = TRUE;

    $project_id = (!empty($node->lingotek_project_id)) ? $node->lingotek_project_id : variable_get('lingotek_project', NULL);
    $source_language = ( isset( $_lingotek_locale[$node->language] ) ) ? $_lingotek_locale[$node->language] : $_lingotek_locale[variable_get( 'lingotek_source_language' )];

    if ($project_id) {
      $parameters = array(
        'projectId' => $project_id,
        'documentName' => $node->title,
        'documentDesc' => $node->title,
        'format' => $this->xmlFormat(),
        'sourceLanguage' => $source_language,
        'tmVaultId' => (!empty($node->lingotek_vault_id)) ? $node->lingotek_vault_id : variable_get('lingotek_vault', 1),
        'content' => lingotek_xml_node_body($node),
        'note' => url('node/' . $node->nid, array('absolute' => TRUE, 'alias' => TRUE))
      );

      $this->addAdvancedParameters($parameters, $node);

      if ($result = $this->request('addContentDocument', $parameters)) {
        lingotek_lingonode($node->nid, 'document_id_' . $node->language, $result->id);
      }
      else {
        $success = FALSE;
      }
    }
    else {
      $success = FALSE;
    }

    return $success;
  }


  /**
   * Adds a target language to an existing Lingotek Document or Project.
   *
   * @param int $lingotek_document_id
   *   The document to which the new translation target should be added.  Or null if the target will be added to the project.
   * @param int $lingotek_project_id
   *   The project to which the new translation target should be added.  Or null if the target will be added to a document instead.
   * @param string $target_language_code
   *   The two letter code representing the language which should be added as a translation target.
   * @param string $workflow_id
   *   The optional workflow to associate with this target. If omitted, the project's default
   *   workflow will be applied.
   *
   * @return mixed
   *  The ID of the new translation target in the Lingotek system, or FALSE on error.
   */
  public function addTranslationTarget($lingotek_document_id, $lingotek_project_id, $target_language_code, $workflow_id = '') {
    global $_lingotek_client, $_lingotek_locale;

    if( isset( $_lingotek_locale[$target_language_code] ) ) {

      $parameters = array(
        'applyWorkflow' => 'true', // Ensure that as translation targets are added, the associated project's Workflow template is applied.
        'targetLanguage' => $_lingotek_locale[$target_language_code]
      );

      if ( isset( $lingotek_document_id ) && !isset( $lingotek_project_id ) ) {
        $parameters['documentId'] = $lingotek_document_id;
      }
      else if ( isset( $lingotek_project_id ) && !isset( $lingotek_document_id ) ) {
        $parameters['projectId'] = $lingotek_project_id;
      }

      if ($workflow_id) {
        $parameters['workflowId'] = $workflow_id;
      }

      if ($new_translation_target = $this->request('addTranslationTarget', $parameters)) {
        return ( $new_translation_target->results == 'success' ) ? TRUE : FALSE ;
      }
      else {
        return FALSE;
      }
    }
    else {
      return FALSE;
    }

  }

  /**
   * Removes a target language to an existing Lingotek Document or Project.
   *
   * @param int $lingotek_document_id
   *   The document to which the new translation target should be added.  Or null if the target will be added to the project.
   * @param int $lingotek_project_id
   *   The project to which the new translation target should be added.  Or null if the target will be added to a document instead.
   * @param string $target_language_code
   *   The two letter code representing the language which should be added as a translation target.
   * @param string $workflow_id
   *   The optional workflow to associate with this target. If omitted, the project's default
   *   workflow will be applied.
   *
   * @return bool
   *  TRUE on success, or FALSE on error.
   */
  public function removeTranslationTarget($lingotek_document_id, $lingotek_project_id, $target_language_code, $workflow_id = '') {
    global $_lingotek_client, $_lingotek_locale;

    if( isset( $_lingotek_locale[$target_language_code] ) ) {

      $parameters = array(
        'applyWorkflow' => 'true', // Ensure that as translation targets are added, the associated project's Workflow template is applied.
        'targetLanguage' => $_lingotek_locale[$target_language_code]
      );

      if ( isset( $lingotek_document_id ) && !isset( $lingotek_project_id ) ) {
        $parameters['documentId'] = $lingotek_document_id;
      }
      else if ( isset( $lingotek_project_id ) && !isset( $lingotek_document_id ) ) {
        $parameters['projectId'] = $lingotek_project_id;
      }

      if ($workflow_id) {
        $parameters['workflowId'] = $workflow_id;
      }

      if ($old_translation_target = $this->request('removeTranslationTarget', $parameters)) {
        return ( $old_translation_target->results == 'success' ) ? TRUE : FALSE ;
      }
      else {
        return FALSE;
      }
    }
    else {
      return FALSE;
    }

  }

  /**
   * Gets the phase data for the active phase of the specified translation target.
   *
   * @param int $translation_target_id
   *   The ID of a translation target on the Lingotek system.
   *
   * @return mixed
   *   An object representing data for the current translation phase, or FALSE on error.
   */
  public function currentPhase($translation_target_id) {
    if ($target = $this->getTranslationTarget($translation_target_id)) {
      if (!empty($target->phases)) {
        $current_phase = FALSE;
        foreach ($target->phases as $phase) {

          if (!$phase->isMarkedComplete) {
            $current_phase = $phase;
            break;
          }
        }

        // Return either the first uncompleted phase, or the last phase if all phases are complete.
        return ($current_phase) ? $current_phase : end($target->phases);
      }
      else {
        return FALSE;
      }
    }
    else {
      return FALSE;
    }
  }

  /**
   * Gets Lingotek Document data for the specified document.
   *
   * @param int $document_id
   *   The ID of the Lingotek Document to retrieve.
   *
   * @return mixed
   *  The API response object with Lingotek Document data, or FALSE on error.
   */
  public function getDocument($document_id) {
    $documents = &drupal_static(__FUNCTION__);

    if (!empty($documents[$document_id])) {
      $document = $documents[$document_id];
    }
    else {
      $params = array('documentId' => $document_id);

      if ($document = $this->request('getDocument', $params)) {
        $documents[$document_id] = $document;
      }
    }

    return $document;
  }

  /**
   * Gets the workflow progress of the specified document.
   *
   * @param int $document_id
   *   The ID of the Lingotek Document to retrieve.
   *
   * @return mixed
   *  The API response object with Lingotek Document data, or FALSE on error.
   */
  public function getDocumentProgress($document_id) {
    $documents = &drupal_static(__FUNCTION__);

    if (!empty($documents[$document_id])) {
      $document = $documents[$document_id];
    }
    else {
      $params = array('documentId' => $document_id);

      if ($document = $this->request('getDocumentProgress', $params)) {
        $documents[$document_id] = $document;
      }
    }

    return $document;
  }

  /**
   * Gets data for a specific Workflow Phase.
   *
   * @param int $phase_id
   *   The ID of the phase to retrieve.
   *
   * @return mixed
   *   The API response object, or FALSE on error.
   */
  public function getPhase($phase_id) {
    $params = array(
      'phaseId' => $phase_id
    );

    return $this->request('getPhase', $params);
  }

  /**
   * Gets a translation target.
   *
   * This fetches an target language object for a specific document.
   *
   * @param int $translation_target_id
   *   ID for the target language object.
   * @return
   *   Object representing a target language for a specific document in the Lingotek platform, or FALSE on error.
   */
  public function getTranslationTarget($translation_target_id) {
    $targets = &drupal_static(__FUNCTION__);

    $params = array(
      'translationTargetId' => $translation_target_id
    );

    if (isset($targets[$translation_target_id])) {
      return $targets[$translation_target_id];
    }
    elseif ($output = $this->request('getTranslationTarget', $params)) {
      $targets[$translation_target_id] = $output;
      return $output;
    }
    else {
      return FALSE;
    }
  }

  /**
   * Gets a workbench URL for the specified document ID and phase.
   *
   * @param int $document_id
   *   A Lingotek Document ID.
   * @param int $phase_id
   *   A Lingotek workflow phase ID.
   *
   * @return mixed
   *   A workbench URL string on success, or FALSE on failure.
   */
  public function getWorkbenchLink($document_id, $phase_id) {
    $links = &drupal_static(__FUNCTION__);

    $static_id = $document_id . '-' . $phase_id;
    if (empty($links[$static_id])) {
      $params = array(
        'documentId' => $document_id,
        'phaseId' => $phase_id,
      );

      if ($output = $this->request('getWorkbenchLink', $params)) {
        $links[$static_id] = $url = $output->url;
      }
      else {
        $url = FALSE;
      }
    }
    else {
      $url = $links[$static_id];
    }

    return $url;
  }

  /**
   * Gets available Lingotek projects.
   *
   * @return array
   *   An array of available projects with project IDs as keys, project labels as values.
   */
  public function listProjects() {
    $projects = array();

    if ($projects_raw = $this->request('listProjects')) {
      foreach ($projects_raw->projects as $project) {
        $projects[$project->id] = $project->name;
      }
    }

    return $projects;
  }

  /**
   * Gets available Lingotek Workflows.
   *
   * @return array
   *   An array of available Workflows with workflow IDs as keys, workflow labels as values.
   */
  public function listWorkflows() {
    $workflows = array();

    if ($workflows_raw = $this->request('listWorkflows')) {
      foreach ($workflows_raw->workflows as $workflow) {
        $workflows[$workflow->id] = $workflow->name;
      }
    }

    return $workflows;
  }

  /**
   * Gets available Lingotek Translation Memory vaults.
   *
   * @return array
   *   An array of available vaults.
   */
  public function listVaults() {
    $vaults = array();

    if ($vaults_raw = $this->request('listTMVaults')) {
      if (!empty($vaults_raw->personalVaults)) {
        foreach ($vaults_raw->personalVaults as $vault) {
          $vaults['Personal Vaults'][$vault->id] = $vault->name;
        }
      }

      if (!empty($vaults_raw->communityVaults)) {
        foreach ($vaults_raw->communityVaults as $vault) {
          $vaults['Community Vaults'][$vault->id] = $vault->name;
        }
      }

      if (!empty($vaults_raw->publicVaults)) {
        foreach ($vaults_raw->publicVaults as $vault) {
          $vaults['Public Vaults'][$vault->id] = $vault->name;
        }
      }
    }

    return $vaults;
  }

  /**
   * Marks a phase as complete.
   *
   * @param int $phase_id
   *   The phase ID to be marked as complete.
   *
   * @return bool
   *   TRUE on success, FALSE on failure.
   */
  public function markPhaseComplete($phase_id) {
    $parameters = array(
      'phaseId' => $phase_id,
    );

    return ($this->request('markPhaseComplete', $parameters)) ? TRUE : FALSE;
  }


  /**
   * Updates the content of an existing Lingotek document with the current node contents.
   *
   * @param stdClass $node
   *   A Drupal node object.
   *
   * @return bool
   *   TRUE on success, FALSE on failure.
   */
  public function updateContentDocument($node) {
    $parameters = array(
      'documentId' => lingotek_lingonode($node->nid, 'document_id_' . $node->language),
      'content' => lingotek_xml_node_body($node),
      'format' => $this->xmlFormat(),
    );

    $this->addAdvancedParameters($parameters, $node);

    return ($this->request('updateContentDocument', $parameters)) ? TRUE : FALSE;
  }

  /**
   * Gets the appropriate format code for the current system state.
   *
   * @return string
   *   A XML format code.
   */
  public function xmlFormat() {
    return (variable_get('lingotek_advanced_parsing', FALSE)) ? 'XML_OKAPI' : 'XML';
  }

  /**
   * Tests the current configuration to ensure that API calls can be made.
   *
   * @return bool
   *   TRUE if the configuration is correct, FALSE otherwise.
   */
  public function testAuthentication() {
    $valid_connection = &drupal_static(__FUNCTION__);

    if (!isset($valid_connection)) {
      $valid_connection = ($this->request('listProjects')) ? TRUE : FALSE;
    }

    return $valid_connection;
  }

  /**
   * Calls a Lingotek API method.
   *
   * @return mixed
   *   On success, a stdClass object of the returned response data, FALSE on error.
   */
  public function request($method, $parameters = array(), $request_method = 'POST') {
    global $user;
    $response_data = FALSE;

    // Almost every v4 API request needs to have the externalID parameter present.
    $parameters += array('externalId' => variable_get('lingotek_login_id', ''));

    module_load_include('php', 'lingotek', 'lib/oauth-php/library/OAuthStore');
    module_load_include('php', 'lingotek', 'lib/oauth-php/library/OAuthRequester');

    $credentials = array(
      'consumer_key' => variable_get('lingotek_oauth_consumer_id', ''),
      'consumer_secret' => variable_get('lingotek_oauth_consumer_secret', '')
    );

    $timer_name = $method . '-' . microtime(TRUE);
    timer_start($timer_name);

    $response = NULL;
    try {
      OAuthStore::instance('2Leg', $credentials);
      $api_url = $this->api_url . '/' . $method;
    	$request = @new OAuthRequester($api_url, $request_method, $parameters);
    	  // There is an error right here.  The new OAuthRequester throws it, because it barfs on $parameters
    	  // The error:  Warning: rawurlencode() expects parameter 1 to be string, array given in OAuthRequest->urlencode() (line 619 of .../modules/lingotek/lib/oauth-php/library/OAuthRequest.php).
    	  // The thing is, if you encode the params, they just get translated back to an array by the object.  They have some type of error internal to the object code that is handling things wrong.
    	  // I couldn't find a way to get around this without changing the library.  For now, I am just supressing the warning w/ and @ sign.
    	$result = $request->doRequest( 0, array( CURLOPT_SSL_VERIFYPEER => FALSE ) );
    	$response = ($method == 'downloadDocument') ? $result['body'] : json_decode($result['body']);
    }
    catch (OAuthException2 $e) {
      watchdog('lingotek', 'Failed OAuth request.
      <br />API URL: @url
      <br />Message: @message. 
      <br />Method: @name. 
      <br />Parameters: !params.
      <br />Response: !response',
        array(
          '@url' => $api_url,
          '@message' => $e->getMessage(),
          '@name' => $method,
          '!params' => $this->watchdog_format_object($parameters),
          '!response' => $this->watchdog_format_object($response)), WATCHDOG_ERROR);
    }

    $timer_results = timer_stop($timer_name);

    if ($this->debug) {
      $message_params = array(
        '@url' => $api_url,
        '@method' => $method,
        '!params' => $this->watchdog_format_object($parameters),
        '!response' => $this->watchdog_format_object($response),
        '@response_time' => number_format($timer_results['time']) . ' ms',
      );

      watchdog('lingotek_debug', '<strong>Called API method</strong>: @method
      <br /><strong>API URL:</strong> @url
      <br /><strong>Response Time:</strong> @response_time<br /><strong>Params</strong>: !params<br /><strong>Response:</strong> !response',
      $message_params, WATCHDOG_DEBUG);
    }

    if ($method == 'downloadDocument' || (!is_null($response) && $response->results == self::RESPONSE_STATUS_SUCCESS)) {
      $response_data = $response;
    }
    else {
      watchdog('lingotek', 'Failed API call.<br />Method: @name. <br />Parameters: !params. <br />Response: !response',
        array('@name' => $method, '!params' => $this->watchdog_format_object($parameters),
        '!response' => $this->watchdog_format_object($response)), WATCHDOG_ERROR);
    }

    return $response_data;
  }

  /**
   * Calls a Lingotek API to provision a new Community (account).
   * Modified version of the request() method.
   *
   * @return mixed
   *   On success, a stdClass object of the returned response data, FALSE on error.
   */
  public function createCommunity( $tag = null, $callback_url = null ) {

    module_load_include('php', 'lingotek', 'lib/oauth-php/library/OAuthStore');
    module_load_include('php', 'lingotek', 'lib/oauth-php/library/OAuthRequester');

    $result = FALSE;
    $response = null;
    $method = 'autoProvisionCommunity';

    // Live Credentials
    $credentials = array('consumer_key' => 'd944c2ae-b66e-4322-b37e-40ba0a495eb7','consumer_secret' => 'e4ae98ca-835b-4d9f-8faf-116ce9c69424');
    // Test Credentials
    // $credentials = array('consumer_key' => '5e6fb433-7a36-4401-8408-413f6592c40d','consumer_secret' => 'f25ec093-e241-4cae-8a78-429cf1248631');

    $parameters = array( );
    //$parameters = array( 'communityDisplayName' => $name );
    if ( isset( $tag ) ) {
      $parameters[ 'distribution' ] = $tag;
    }
    if ( isset( $callback_url ) ) {
      $parameters[ 'callbackUrl' ] = $callback_url . '?doc_id={document_id}&target_code={target_language}&project_id={project_id}';
    }

    $timer_name = $method . '-' . microtime(TRUE);
    timer_start($timer_name);

    try {
      OAuthStore::instance('2Leg', $credentials);
    	$request = new OAuthRequester(  $this->api_url . '/autoProvisionCommunity', 'POST', $parameters);
    	$result = $request->doRequest( 0, array( CURLOPT_SSL_VERIFYPEER => FALSE ) );
    	watchdog( 'lingotek_community', 'Provision Community: !result', array( '!result' => watchdog_format_object( $result ) ), WATCHDOG_DEBUG );
    }
    catch (OAuthException2 $e) {
      watchdog('lingotek', 'Failed OAuth request.
      <br />Message: @message. <br />Method: @name. <br />Parameters: !params. <br />Response: !response',
        array('@message' => $e->getMessage(), '@name' => $method, '!params' => $this->watchdog_format_object($parameters),
        '!response' => $this->watchdog_format_object($response)), WATCHDOG_ERROR);      
    }

    $timer_results = timer_stop($timer_name);

    if ($this->debug) {
      $message_params = array(
        '@method' => $method,
        '!params' => $this->watchdog_format_object($parameters),
        '!response' => $this->watchdog_format_object($response),
        '@response_time' => number_format($timer_results['time']) . ' ms',
      );
      watchdog(
        'lingotek_debug',
        '<strong>Called API method</strong>: @method<br />
        <strong>Response Time:</strong> @response_time<br />
        <strong>Params</strong>: !params<br />
        <strong>Response:</strong> !response',
        $message_params,
        WATCHDOG_DEBUG
      );
    } // END:  if debug

    return $result;
  }


  /**
   * Get Account Status
   * Request:  http://cp.lingotek.com:8080/billing/account.json?community=B2MMD3X5&external_id=community_admin&oauth_key=28c279fa-28dc-452e-93af-68d194a2c366&oauth_secret=0e999486-3b4d-47e4-ba9a-d0f3f0bbda73
   * Response:  {"state":"active","plan":{"trial_ends_at":0,"state":"active","activated_at":1355267936,"type":"cosmopolitan_monthly","languages_allowed":2,"language_cost_per_period_in_cents":14900}}
   * Will return FALSE or a json decoded object.
   */
  function getAccountStatus( ) {

    $result = FALSE;
    $fields = array(
      'community'    => urlencode( variable_get( 'lingotek_community_identifier', '' ) ),
      'external_id'  => urlencode( variable_get( 'lingotek_login_id', '' ) ),
      'oauth_key'    => urlencode( variable_get( 'lingotek_oauth_consumer_id', '' ) ),
      'oauth_secret' => urlencode( variable_get( 'lingotek_oauth_consumer_secret', '' ) ),
    );
    //dpm( $fields );

    if( ( isset($fields['community']) && $fields['community'] != '' ) && ( isset($fields['external_id']) && $fields['external_id'] != '' )  && ( isset($fields['oauth_key']) && $fields['oauth_key'] != '' )  && ( isset($fields['oauth_secret']) && $fields['oauth_secret'] != '') ) {

      $url = LINGOTEK_BILLING_SERVER . '?';
      foreach( $fields as $key=>$value ) { 
        $url .= $key . '=' . $value . '&';
      }
      $url = substr( $url, 0, -1 );

      $ch = curl_init( $url );
      curl_setopt( $ch, CURLOPT_RETURNTRANSFER, TRUE );
      curl_setopt( $ch, CURLOPT_SSL_VERIFYPEER, FALSE );
      //curl_setopt( $ch, CURLINFO_HEADER_OUT, TRUE );
      $response = curl_exec( $ch );
      $info = curl_getinfo( $ch );
      curl_close( $ch );

      $json = json_decode( $response );
      if ( isset( $json ) ) { // Did we get valid data back?  If not, $json is NULL.

        //dpm( '- - - - - - - - - - - - - - -' );
        //dpm( 'URL: ' . $url );
        //dpm( 'INFO:' );
        //dpm( $info );
        //dpm( 'RESULT (json):' );
        //debug( $json );
        //dpm( '- - - - - - - - - - - - - - -' );

        $result = $json;

      }

    } // END:  has credentials

    return $result;

  } // END:  lingotek_get_account_status()


  /**
   * Formats a complex object for presentation in a watchdog message.
   */
  private function watchdog_format_object($object) {
    return '<pre>' . htmlspecialchars(var_export($object, TRUE)) . '</pre>';
  }

  /**
   * Adds advanced parameters for use with addContentDocument and updateContentDocument.
   *
   * @param array $parameters
   *   An array of API request parameters.
   * @param object $node
   *   A Drupal node object.
   */
  private function addAdvancedParameters(&$parameters, $node) {
    // Extra parameters when using advanced XML configuration.
    $advanced_parsing_enabled = variable_get('lingotek_advanced_parsing', FALSE);
    $use_advanced_parsing = ($advanced_parsing_enabled ||
      (!$advanced_parsing_enabled && lingotek_lingonode($node->nid, 'use_advanced_parsing')));

    if ($use_advanced_parsing) {
      $advanced_parameters = array(
        'fprmFileContents' => variable_get('lingotek_advanced_xml_config1',''),
        'secondaryFprmFileContents' => variable_get('lingotek_advanced_xml_config2',''),
        'secondaryFilter' => 'okf_html',
      );

      $parameters = array_merge($parameters, $advanced_parameters);
    }
  }

  /**
   * Private constructor.
   */
  private function __construct() {
    $this->debug = variable_get('lingotek_api_debug', FALSE);
    $host = variable_get('lingotek_url', self::LINGOTEK_SERVER_PRODUCTION);
    // Trim trailing slash from user-entered server name, if it exists.
    if (substr($host, -1) == '/') {
      $host = substr($host, 0, -1);
    }    
    $this->api_url = $host . self::API_ENDPOINT_V4;
  }

  /**
   * Private clone implementation.
   */
  private function __clone() {}

}
